<?php

App::uses('AppComponent', 'Controller/Component');
App::uses('CakeText', 'Utility');
App::uses('InflectorExt', 'EvInflector.Lib');

class MetaComponent extends AppComponent {

/**
 * Default settings for this Component
 *
 * @var array
 */
	protected $_defaultSettings = [
		'descriptionField' => 'content', // The field from the model to use to set the meta description
		'descriptionLength' => 200 // The length of the meta description when using model fields
	];

	/**
	 * Sets the view variables for the meta data.
	 *
	 * @param array $data array containing the MetaData
	 * @param string $overrideModel The main model, useful if setting Page content from within a plugin
	 */
	public function set($data, $overrideModel = null) {
		$Model = $this->_controller->{$this->_controller->modelClass};

		// override the Model, otherwise plugins will look at their own models
		// for the title when it may actually want to look at 'Page' Model.
		if (! empty($overrideModel)) {
			$this->_controller->loadModel($overrideModel);
			list($pluginName, $modelName) = pluginSplit($overrideModel);
			$Model = $this->_controller->{$modelName};
		}

		$this->_createSettings($Model);

		// Set the title for layout, either using the meta title, display
		// field, or model name.
		$title = $this->_getTitleForLayout($data, $Model);

		$description = $this->_getDescriptionForLayout($data, $Model);

		$keywords = $this->_getKeywordsForLayout($data, $Model);

		$this->_controller->set('titleForLayout', $title);

		// Set title_for_layout for backwards compatibility (CakePhp 2.4.x).
		$this->_controller->set('title_for_layout', $title);

		$this->_controller->set('metaTitle', $title);
		$this->_controller->set('metaDescription', $description);
		$this->_controller->set('metaKeywords', $keywords);

		return;
	}

	/**
	 * Create the component settings based on the default setting, the model fields and any passed settings.
	 * @param  object $Model The model to check which fields are available for accessing the description
	 */
	protected function _createSettings($Model) {
		if ($Model->hasField('body')) {
			$defaultSettings['descriptionField'] = 'body';
		} else {
			$defaultSettings['descriptionField'] = $Model->hasField('description') ? 'description' : 'content';
		}

		$this->settings = array_merge($this->_defaultSettings, $defaultSettings, $this->settings);
	}

	/**
	 * Set the title for layout, either using the meta title, display field, or model name.
	 * @param  array  $data  The data passed through from the controller
	 * @param  object $Model The model to check available fields
	 * @return string        The meta title to set
	 */
	protected function _getTitleForLayout($data, $Model) {
		if (!empty($data['MetaData']['title'])) {
			$titleForLayout = $data['MetaData']['title'];
		} elseif (isset($data[$Model->alias][$Model->displayField])) {
			$titleForLayout = $data[$Model->alias][$Model->displayField] . ' | ' . Configure::read('SiteSetting.general.site_title');
		} else {
			$titleForLayout = InflectorExt::pluralize($Model->displayName) . ' | ' . Configure::read('SiteSetting.general.site_title');
		}

		return $titleForLayout;
	}

	/**
	 * Set the description for layout, either using the meta description ordescription field.
	 * @param  array  $data  The data passed through from the controller
	 * @param  object $Model The model to check available fields
	 * @return string        The meta description to set
	 */
	protected function _getDescriptionForLayout($data, $Model) {
		if (!empty($data['MetaData']['description'])) {
			$descriptionForLayout = $data['MetaData']['description'];
		} elseif (isset($data[$Model->alias][$this->settings['descriptionField']])) {
			$descriptionForLayout = CakeText::Truncate(strip_tags($data[$Model->alias][$this->settings['descriptionField']]), $this->settings['descriptionLength']);
		} else {
			$descriptionForLayout = null;
		}

		return $descriptionForLayout;
	}

	/**
	 * Set the description for layout, using the meta keywords or null.
	 * @param  array  $data  The data passed through from the controller
	 * @param  object $Model The model to check available fields
	 * @return string        The meta keywords to set
	 */
	protected function _getKeywordsForLayout($data, $Model) {
		if (!empty($data['MetaData']['keywords'])) {
			$keywordsForLayout = $data['MetaData']['keywords'];
		} else {
			$keywordsForLayout = null;
		}

		return $keywordsForLayout;
	}
}
